#!/usr/bin/php
<?php

#################################################################
# This script is called by openvpn daemon whenever a client 
# connects (--client-connect parameter in ovpn config file).   
#
# Client's ovpn_cn and real_ip are passed to this script 
# through environmental variables 'common_name' and 'trusted_ip'.
#
# More details on this script can be found in 'man openvpn'.
# Exit value should be 0 to let the connection be established.

function logger($facility, $msg)
{
  openlog("CustomerServer", LOG_PID | LOG_NDELAY, LOG_LOCAL0);
  syslog($facility, $msg);  
}

# We must use 'cert_serial' instead of 'common_name', because
# otherwise there is confusion about routers' identity after
# CS is reinstalled with import of previous certificates. 
$cert_serial = getenv('tls_serial_0');
$real_ip     = getenv('trusted_ip');

$phpclientapi = new CSPhpClientApi();

try {
  $device = $phpclientapi->get_device_by_cert_serial($cert_serial);
  
  if ($device['validated'] != "1") {
    throw new Exception('Device is not validated.');
  }
  if ($device['disabled'] != "0") {
    throw new Exception('Device is disabled.');
  }
  if ($device['device_type'] != "2") { # 1 = RW, 2 = Router
    throw new Exception('Wrong device type [' . $device['device_type'] . '].');
  }
  $phpclientapi->set_router_online($device['device_id'], $real_ip);
  $phpclientapi->add_connlog_connect($device['device_id']);
} catch (Exception $exception) {
  logger(LOG_DEBUG, "DENIED OpenVPN access to router ($real_ip): " . $exception->getMessage());
  exit(1);
}
