#!/usr/bin/python3

import os, string, random

from shutil         import copyfile

from functions      import g_file_apache2_conf_tmp, g_file_apache2_conf, g_file_cs_settings, g_file_apache2_conf, g_file_cs_db
from functions      import g_file_proxy_single_conf, g_file_proxy_template, g_file_proxy_multi_conf, g_dir_apache2_proxy
from functions      import exec_command
from database       import SettingsDB, Database
from vpn_constants  import *

####################################################################
# Delete file without raising an exception if the file doesn't exist.
def delete_file(fname):
  try:
    os.remove(fname)
  except:
    pass

####################################################################
# Create a real apache config from a template by adding the proper
# define.
def create_apache2_config():
  c_apache2_variables_start = 'VARIABLES:START'
  with open(g_file_apache2_conf_tmp, "r") as conf_tmp_r:
    with open(g_file_apache2_conf, "w") as conf_w:
      for line in conf_tmp_r:
        conf_w.write(line)
        if c_apache2_variables_start in line:
          s = SettingsDB(g_file_cs_settings)
          conf_w.write('  define CS_DOMAIN_NAME ' + s.get_value("APACHE2_PROXY_DOMAIN_TOP") + '\n')

####################################################################
# 
def delete_dir_contents(directory):
  for filename in os.listdir(directory):
    abs_fpath = os.path.join(directory, filename)
    os.remove(abs_fpath)
    
####################################################################
# Generate random string of the given length, consisting of digits  
# and lower case characters.
def random_str(length):
    letters = string.ascii_lowercase + string.digits
    return ''.join(random.sample(letters,length))   
    
####################################################################
# URL example: 10-8-1-1-ymt1xuj7z132i454unub.ondra.vpnportal.test
def form_proxy_url(ovpn_ip, domain_name):
  rnd_part_length = 20
  return ovpn_ip.replace(".", "-") + "-" + random_str(rnd_part_length) + "." + domain_name

####################################################################
# Writes proxy_url to DB for each Standalone client.
def db_generate_rw_proxy_urls():
  db = Database(g_file_cs_db)
  st = SettingsDB(g_file_cs_settings)
  rws         = db.run('SELECT device_id, ovpn_ip FROM device_table WHERE device_type="1";') # device_type: 1 = ROADWARRIOR, 2 = ROUTER
  domain_name = st.get_value("APACHE2_PROXY_DOMAIN_TOP")
  for rw in rws:
    proxy_url = form_proxy_url(rw[1], domain_name)
    db.run('UPDATE device_table SET proxy_url="'+proxy_url+'" WHERE device_id="'+str(rw[0])+'";')

####################################################################
# Insert configuration of direct proxies into 'single.conf' file.
def update_single_conf():
  db = Database(g_file_cs_db)
  entries = db.run('SELECT proxy_url, ovpn_ip FROM device_table WHERE validated="1";')
  proxies_txt = ""
  for row in entries:
    proxies_txt += "Use EnableProxy " + row[0] + " " + row[1] + "\n";
  with open(g_file_proxy_single_conf, 'w') as f:
    f.write(proxies_txt)

####################################################################
####################################################################
          
print('Migrating apache config from 1.0.1 to 1.1.0')

# Move the new template to apache's dir.
copyfile("resources/vpnportal-ssl.conf.template", g_file_apache2_conf_tmp)

# Create a real apache config from a template by adding
# the proper defines.
create_apache2_config()

# Enable apache's mode 'macro'.
exec_command(["a2enmod", "macro"])

# Erase all files in proxy dir.
delete_dir_contents(g_dir_apache2_proxy)

# Copy new files to proxy dir.
copyfile("resources/proxy.conf.template", g_file_proxy_template)
copyfile("resources/single.conf", g_file_proxy_single_conf) 
copyfile("resources/multi.conf", g_file_proxy_multi_conf) 

# Generate new proxy URLs for Standalone clients and write them to DB.
db_generate_rw_proxy_urls()

# Insert configuration of direct proxies into 'single.conf' file.
update_single_conf()



