#!/usr/bin/python3

# This script shall be used during update from 1.0.1 to 1.1.0 version
# of vpnportal to migrate the web database.
#
# It needs to do the following:
#     * Rename roles
#     * Remove all roles except the highest one.
#     * Update 's:' value in the column to reflect the new length of the name. 
#

from   functions   import g_file_web_db
from   db_migrator import DBMigrator

####################################################################
# Example of the value in 'roles' column:
# 'a:4:{i:0;s:17:"ROLE_ROUTER_ADMIN";i:1;s:18:"ROLE_NETWORK_ADMIN";i:2;s:20:"ROLE_NETWORK_MANAGER";i:3;s:19:"ROLE_SYSTEM_MANAGER";}'

####################################################################
# This class represents one entry in 'role' column of the web 
# database.
class RoleColEntry():
  def __init__(self, text):
    self.init_from_text(text)

  # Parameter 'text' should contain something looking 
  # like this: i:0;s:17:"ROLE_ROUTER_ADMIN"
  def init_from_text(self, text):
    text = text[:-1]
    index_s, name_s = text.split(';')
    self.index = index_s.split(':')[0]
    self.size  = name_s.split(':')[1]
    self.name  = name_s.split(':')[2][1:-1] # Strip quotes from the name
  
  def set_name(self, name):
    self.size = len(name)
    self.name = name
    
  def set_index(self, index):
    self.index = index

  def to_str(self):
    return 'i:'+str(self.index)+';s:'+str(self.size)+':"'+self.name+'";'
  
####################################################################
# Parameter 'text' should contain one entry from 'role' column of 
# web database. It could look for example like this:
# 'a:2:{i:0;s:17:"ROLE_ROUTER_ADMIN";i:1;s:18:"ROLE_NETWORK_ADMIN";}'
def extract_roles(text):
  roles = []
  # Strip out what we dont need.
  text = text[text.find('{')+1 : text.find('}')]
  i = 1
  word = ""
  for s in text.split(';'):
    word += s + ";"
    if i % 2 == 0:
      roles.append(RoleColEntry(word))
      word = ""
    i = i+1
  return roles

####################################################################
# Returns True if the first role is higher than the second one.
def is_higher_role(role1, role2):
  role_order = [ 'ROLE_OBSERVER', 'ROLE_DEVICE_ADMIN', 'ROLE_NET_MANAGER', 'ROLE_NET_ADMIN', 'ROLE_SYSTEM_ADMIN' ]
  return role_order.index(role1) > role_order.index(role2)

####################################################################
# 
def get_highest_role(roles):
  best_index = 0
  i = 0
  for role in roles:
    # Returns True if the first role is higher than the second one.
    if is_higher_role(role.name, roles[best_index].name):
      best_index = i
    i = i+1
  return roles[best_index]

####################################################################
# Takes a value from the column and returns updated value.
def functor(old_value):
  roles = extract_roles(old_value)
  # Rename roles
  for role in roles:
    if role.name == 'ROLE_VIEWER':
      role.set_name('ROLE_OBSERVER')
    elif role.name == 'ROLE_ROUTER_ADMIN':
      role.set_name('ROLE_DEVICE_ADMIN')
    elif role.name == 'ROLE_NETWORK_ADMIN':
      role.set_name('ROLE_NET_ADMIN')
    elif role.name == 'ROLE_NETWORK_MANAGER':
      role.set_name('ROLE_NET_MANAGER')
    elif role.name == 'ROLE_SYSTEM_MANAGER':
      role.set_name('ROLE_SYSTEM_ADMIN')
  winner = get_highest_role(roles)
  winner.set_index(0)
  return '\'a:1:{'+winner.to_str()+'}\''
  
####################################################################
####################################################################

print('Migrating WEB database from 1.0.1 to 1.1.0')

dbm = DBMigrator(g_file_web_db)

# Replaces all values in column 'roles' with values returned 
# by functor.
dbm.update_col_values_f('app_user', 'roles', functor)

# Commit the changes.
dbm.commit()



