#!/usr/bin/python3

# This script shall be used during update from 1.0.1 to 1.1.0 version
# to update values of 'ovpn_cert_serial' column in DB.
#

import os
from   OpenSSL     import crypto

from   functions   import g_file_cs_db, g_dir_rw_cli_certs, g_dir_router_cli_certs
from   db_migrator import DBMigrator

####################################################################
# Returns an associative array where keys are device_ids and values
# are OVPN certificate serial numbers.
def get_serials(cert_directory):
  serials = dict()
  # Iterate over files.
  for filename in os.listdir(cert_directory):
    if filename.endswith(".crt"): 
      abs_fpath = os.path.join(cert_directory, filename)
      device_id = extract_cert_cn(abs_fpath)
      serials[device_id] = extract_cert_serial(abs_fpath)
  return serials

####################################################################
# Returns certificate's serial number as a string.
def extract_cert_serial(cert_file):
  cert = crypto.load_certificate(crypto.FILETYPE_PEM, open(cert_file).read())
  return str(cert.get_serial_number())

####################################################################
# Returns certificate's serial number as a string.
def extract_cert_cn(cert_file):
  cert = crypto.load_certificate(crypto.FILETYPE_PEM, open(cert_file).read())
  return cert.get_subject().CN
  
####################################################################
# Writes certificate serial numbers into the database.
def update_db_serials(serials_map):
  for device_id, serial in serials_map.items():
    dbm.db.run("update device_table set ovpn_cert_serial = '" + serial + "' where device_id =\""+ device_id +"\";")

####################################################################
####################################################################

print('Updating OVPN certificate serials in DB.')

dbm = DBMigrator(g_file_cs_db)

# Get map (associative array) where keys are device_ids and values are 
# OVPN cert serials.
rw_serials   = get_serials(g_dir_rw_cli_certs)
#ovpn_serials = get_serials(g_dir_router_cli_certs)

update_db_serials(rw_serials)
#update_db_serials(ovpn_serials)

