#!/usr/bin/python3

# This script must be called before database is migrated.
# Its purpose is to revoke keys of all roadwarrior clients.

import os

from functions import g_file_cs_db, g_dir_ca, g_dir_rw_cli_certs
from database  import Database

####################################################################
# Create a temporary config file in 'out_file'.
def create_openssl_config(out_file):
  s = '''[ca]
         default_ca = ca_default
           
         [ca_default]
         certificate      = /opt/vpnportal/customer_server/certificates/ca/ca.crt
         private_key      = /opt/vpnportal/customer_server/certificates/ca/ca.pem
         database         = /opt/vpnportal/customer_server/certificates/ca/index.txt
         serial           = /opt/vpnportal/customer_server/certificates/ca/serial.txt
         default_md       = sha256
         default_crl_days = 9999'''
  f = open(out_file, "w")
  f.write(s)

####################################################################
# Print a message and exit with error.
def die(msg):
  print(msg)
  exit(1)

####################################################################
####################################################################

tmp_cfg_path = "/tmp/openssl.cfg"

db = Database(g_file_cs_db)

# Prepare config file for revocation.
create_openssl_config(tmp_cfg_path)

# In 1.0.1 roadwarrior's common name was stored in 'rw_name' column.
rows = db.run('select rw_name from roadwarrior_table;')

# Revoke each roadwarrior (this only updates index file, not the actual CRL)
for row in rows:
  rw_fname = row[0].replace(" ", "_") 
  rw_cert  = g_dir_rw_cli_certs + "/" + rw_fname + ".crt"
  rw_key   = g_dir_rw_cli_certs + "/" + rw_fname + ".pem"
  if os.system("sudo openssl ca -config " + tmp_cfg_path + " -revoke " + rw_cert + " -crl_reason superseded ") != 0:
    die("Failed to revoke certificate for client "+row[0])
  os.remove(rw_cert)
  os.remove(rw_key)

# Update revocation list (this synchronizes CRL with the index file that was changed in the previous step)
if len(rows) > 0:
  crl = g_dir_ca + "/revoked_certs.crl"
  if os.system("sudo openssl ca -batch -config " + tmp_cfg_path + " -gencrl -out " + crl) != 0:
    die("Failed to update certificate revocation list.")

# Clean up
os.remove(tmp_cfg_path)
