#!/usr/bin/python3

# This script must be called before database is migrated.
# The purpose of this script is to update values in DB 
# column 'expiration_days', so that it would contain the
# date of expiration (in seconds since the beginning of the
# epoch) instead of number of remaining days. This column 
# will be renamed to 'expiration_date' in a different script.
# 

import subprocess
import time
import datetime

from OpenSSL   import crypto
from os        import system

from functions import g_file_cs_db, g_dir_rw_cli_certs
from database  import Database

####################################################################
# Returns expiration time of the certificate as 'YYYYMMDDhhmmssZ'
# string.
def get_valid_until(cert_file):
  cert = crypto.load_certificate(crypto.FILETYPE_PEM, open(cert_file).read())
  return str(cert.get_notAfter().decode("utf-8"))

####################################################################
####################################################################

db = Database(g_file_cs_db)

rows = db.run('select rw_name from roadwarrior_table;')

# For each standa:
for row in rows:
  rw_fname = row[0].replace(" ", "_") 
  rw_cert = g_dir_rw_cli_certs + "/" + rw_fname + ".crt"
  valid_until = get_valid_until(rw_cert) # Returns date in this format: YYYYMMDDhhmmssZ
  year  = int(valid_until[:4])
  month = int(valid_until[4:6])
  day   = int(valid_until[6:8])
  hour  = int(valid_until[8:10])
  t = datetime.datetime(year, month, day, hour, minute=0)
  expiration_in_epoch_s = time.mktime(t.timetuple())
  db.run("UPDATE roadwarrior_table set expiration_days='" + str(expiration_in_epoch_s) + "' WHERE rw_name='"+row[0]+"'")
  
