<?php

namespace AppBundle\Controller\Ajax;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class AjaxFwRulesController
 * @package AppBundle\Controller\Ajax
 */
class AjaxFwRulesController extends AjaxBaseController
{

    /**
     * Render Ajax page.
     *
     * @Route("ajax/device-fw-rules", name="app_ajax_device_fw_rules_table")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxDeviceFwRulesAction(Request $request){

        //If is not ajax
        if(!$request->isXmlHttpRequest()){

            return new Response('Bad Request', 400);

        }
        $aOrder = array();

        list($start, $length, $aOrder, $filter) = $this->parseDeviceData($request);

        $filter['validated'] = '1';

        $devices = $this->apiDevice->getAllDevices($start, $length, $aOrder, $filter);

        $aDeviceRules = array();

        $deviceCount = 0;

        $sortingArray = array();

        if($devices){

            foreach ($devices as $device){

                $sortingArray[] = $device['device_id'];

                $incomingDefaultPolicy = $this->apiFwRules->getDeviceFwPolicy($device['device_id'], $this->apiFwRules::TRAFFIC_DIRECTION_INCOMING);

                $where_condition['related_policy'] = $incomingDefaultPolicy;    // ACCEPT=0, DROP=1
                $where_condition['traffic_direction'] = $this->apiFwRules::TRAFFIC_DIRECTION_INCOMING; // INCOMING=1, OUTGOING=2

                $fwRules = $this->apiFwRules->getDeviceFwRules($device['device_id'], $where_condition);

                foreach ($fwRules as $fwRule){

                    $fwRule = array_merge(array(
                        'id' => $device['device_id'],
                        'name' => $device['name'],
                        'policy' => (($incomingDefaultPolicy == 0) ? 'Incoming: Allow' : 'Incoming: Deny'),
                        'action' => (($incomingDefaultPolicy == 0) ? 'Allow' : 'Deny')
                    ), $fwRule);

                    $aDeviceRules[$device['device_id']][] = $fwRule;

                }

            }

            foreach ($devices as $device){

                $outgoingDefaultPolicy = $this->apiFwRules->getDeviceFwPolicy($device['device_id'], $this->apiFwRules::TRAFFIC_DIRECTION_OUTGOING);

                $where_condition['related_policy'] = $outgoingDefaultPolicy;    // ACCEPT=0, DROP=1
                $where_condition['traffic_direction'] = $this->apiFwRules::TRAFFIC_DIRECTION_OUTGOING; // INCOMING=1, OUTGOING=2

                $fwRules = $this->apiFwRules->getDeviceFwRules($device['device_id'], $where_condition);

                foreach ($fwRules as $fwRule){

                    $fwRule = array_merge(array(
                        'id' => $device['device_id'],
                        'name' => $device['name'],
                        'policy' => (($outgoingDefaultPolicy == 0) ? 'Outgoing: Allow' : 'Outgoing: Deny'),
                        'action' => (($outgoingDefaultPolicy == 0) ? 'Drop' : 'Allow')
                    ), $fwRule);

                    $aDeviceRules[$device['device_id']][] = $fwRule;

                }

            }

            //sort by original order
            $result = array();
            foreach ($sortingArray as $id) {
                if ( array_key_exists($id, $aDeviceRules) ) {
                    $result[$id] = $aDeviceRules[$id];
                }
            }

            $aDeviceRules = $result;
            $deviceCount = count($aDeviceRules);
            $aDeviceRulesTemp = array();

            foreach($aDeviceRules as $key => $rule){

                if(is_array($rule)){

                    foreach ($rule as $item){

                        $aDeviceRulesTemp[] = $item;

                    }

                }else{

                    $aDeviceRulesTemp[] = $rule;

                }

            }

            $aDeviceRules = $aDeviceRulesTemp;

            if(empty($aDeviceRules)){

                $aDeviceRules = array();

            }

        }

        $data =  [

            "recordsTotal" => $deviceCount,
            "recordsFiltered" => $deviceCount,
            "data" => $aDeviceRules

        ];

        return new JsonResponse($data);

    }

    /**
     * Render Ajax page.
     *
     * @Route("ajax/network-fw-rules", name="app_ajax_network_fw_rules_table")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function ajaxNetworkFwRulesAction(Request $request){

        //If is not ajax
        if(!$request->isXmlHttpRequest()){

            return new Response('Bad Request', 400);

        }

        list($start, $length, $aOrder, $filter) = $this->parseNetworkData($request);

        $networks = $this->apiDevice->getAllNetworks($start, $length, $aOrder, $filter);

        $aNetworkRules = array();

        if(!empty($networks)){

            foreach ($networks as $network){

                $defaultPolicy = $this->apiFwRules->getNetworkFwPolicy($network['network_id']);

                $where_condition['related_policy'] = $defaultPolicy;    // ACCEPT=0, DROP=1

                $fwRules = $this->apiFwRules->getNetworkFwRules($network['network_id'], $where_condition);

                if(!empty($fwRules)){

                    foreach ($fwRules as $fwRule){

                        $data = array(

                            'id' => $network['network_id'],
                            'name' => $network['name'],
                            'policy' => (($defaultPolicy == 0) ? 'Allow' : 'Deny'),

                        );

                        $fwRule = array_merge($data, $fwRule);

                        $aNetworkRules[] = $fwRule;

                    }

                }

            }

        }

        $data =  [

            "recordsTotal" => count($aNetworkRules),
            "recordsFiltered" => count($aNetworkRules),//$dbApi->getCountOfAllNetworks($filter),
            "data" => $aNetworkRules

        ];

        return new JsonResponse($data);

    }

    /**
     * @param $request
     * @return array
     */
    private function parseDeviceData($request){

        $columns = array(

            0 => 'name',
            1 => 'name',

        );

        return $this->parseData($request, $columns);

    }

    /**
     * @param $request
     * @return array
     */
    private function parseNetworkData($request){

        $columns = array(

            0 => 'name',

        );

        return $this->parseData($request, $columns);

    }

    /**
     * @param $request
     * @return array
     */
    private function parseData($request, array $columns){

        $parametersToValidate = $request->query->all();

        $start = $request->query->getInt('start');
        $length = $request->query->getInt('length');

        // Individual column filtering
        $filterColumns = $request->query->get('columns');
        $search = $request->query->get('search');

        $filter = array();

        if(isset($search['value']) && !empty($search['value'])){

            $filter[$columns[ 0 ]] = '%' . $search['value'] . '%';

        }else{

            foreach ($filterColumns as $key => $column){

                $columnIdx = array_search( $column['data'], $columns );

                if($columnIdx && $column['search']['value'] != ''){

                    $filter[$columns[ $columnIdx ]] = '%' . $column['search']['value'] . '%';

                }

            }

        }

        $order = $request->query->get('order');
        $aOrder = array();

        if($order){

            foreach ($order as $item){

                $aOrder[$columns[$item['column']]] = $item['dir'];
            }

        }

        return array($start, $length, $aOrder, $filter);

    }

}