<?php

namespace AppBundle\Controller;

use AppBundle\Api\Device;
use AppBundle\Api\FwRules;
use AppBundle\Entity\NetworkFwRule;
use AppBundle\Entity\NetworkFwRules;
use AppBundle\Entity\User;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use AppBundle\Form\Network;

/**
 * Class NetworksController
 * @package AppBundle\Controller
 */
class NetworksController extends BaseController
{

    /**
     * @Route("/networks", name="app_manage_networks")
     *
     * @param Request $request Request
     *
     * @return Response
     */
    public function manageNetworksAction(Request $request){

        return $this->render('app/pages/network/manageNetworks.html.twig', []);

    }

    /**
     * @Route("/network/add", options={"expose"=true}, name="app_add_network")
     *
     * @param Request $request Request
     *
     * @return Response
     */
    public function addNetworkAction(Request $request){

        $this->denyAccessUnlessGranted(User::ROLE_NET_ADMIN);

        $network = $this->container->get('network');

        $form = $this->createForm(Network::class, $network, array('isNew' => true));

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

            $apiDevice = $this->container->get('cs_api_device');

            $result = $apiDevice->addNetwork($network->getName());

            if($result === true){

                $this->addFlash(
                    'notice',
                    'Network added!'
                );

            }else{

                $this->addFlash(
                    'error',
                    $result
                );

            }

            return $this->redirectToRoute('app_manage_networks');
        }

        return $this->render('app/pages/network/networkEdit.html.twig', ['title' => 'Add Network', 'form' => $form->createView()]);

    }

    /**
     * @Route("/network/detail/{id}", options={"expose"=true}, name="app_network_detail")
     *
     * @param Request $request Request
     *
     * @return Response
     */
    public function networkDetail(Request $request, $id){

        /** @var Device $apiDevice */
        $apiDevice = $this->container->get('cs_api_device');

        $network = $apiDevice->getNetworkById($id);

        if($network){

            $networkName = $network['name'];

        }

        $devices = $apiDevice->getDevicesByNetworkId($id, 0, 1000);

        return $this->render('app/pages/network/manageDevicesInNetwork.html.twig',
                ['networkId' => $id, 'networkName' => $networkName]);

    }

    /**
     * @Route("network/edit/{id}", options = { "expose" = true }, name="app_edit_network")
     *
     * @param Request $request Request
     *
     * @return Response
     */
    public function editNetworkAction(Request $request, $id){

        $this->denyAccessUnlessGranted(User::ROLE_NET_MANAGER);

        /** @var Device $apiDevice */
        $apiDevice = $this->container->get('cs_api_device');

        $networkRenamed = 1;

        /** @var FwRules $apiFwRules */
        $apiFwRules = $this->container->get('cs_api_fw_rules');

        $result = $apiDevice->getNetworkById($id);

        if(!$result){

            $this->addFlash(
                'error',
                'Error while reading network detail!'
            );

            return $this->redirectToRoute('app_manage_networks');

        }

        $where_condition['related_policy'] = FwRules::RELATED_POLICY_DROP; // ACCEPT=0, DROP=1
        $acceptFwRules = $apiFwRules->getNetworkFwRules($id, $where_condition);
        $acceptFwRules = $this->setFwRules($id, $acceptFwRules);

        $where_condition['related_policy'] = FwRules::RELATED_POLICY_ACCEPT; // ACCEPT=0, DROP=1
        $denyFwRules = $apiFwRules->getNetworkFwRules($id, $where_condition);
        $denyFwRules = $this->setFwRules($id, $denyFwRules);

        $defaultFwPolicy = $apiFwRules->getNetworkFwPolicy($id);

        $network = $this->container->get('network');
        $network->setId($id);
        $network->setName($result['name']);
        $network->setDefaultFwPolicy($defaultFwPolicy);
        $network->setAcceptFwRules($acceptFwRules);
        $network->setDenyFwRules($denyFwRules);

        $networkDb = clone $network;

        $form = $this->createForm(Network::class, $network, array('isNew' => false));

        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {

            $toUpdateArray = array_diff($network->getData(), $networkDb->getData());

            foreach ($toUpdateArray as $type => $value) {

                if ($type == 'name') {

                    $networkRenamed = $apiDevice->updateNetwork($id, $value);

                }

            }

            try {

                $this->setChangedFwRules($networkDb, $network);

            } catch (\Exception $e) {

                $errorMessage = $e->getMessage();

            }

            if ('apply' === $form->getClickedButton()->getName()) {

                if (isset ($errorMessage)) {
                    $this->addFlash(
                        'error',
                        $errorMessage . ', no FW rule was set.'
                    );
                }elseif ($networkRenamed != 1) {
                    $this->addFlash(
                        'error',
                        'Network name is already taken. FW rules were saved.'
                    );
                } else{

                    $this->addFlash(
                        'notice',
                        'Your changes were saved!'
                    );

                }

                return $this->redirect($request->getUri());

            } else {

                if (isset ($errorMessage)) {

                    $this->addFlash(
                        'error',
                        $errorMessage . ', no FW rule was set.'
                    );

                }elseif ($networkRenamed != 1) {
                    $this->addFlash(
                        'error',
                        'Network name is already taken. FW rules were saved.'
                    );
                } else{

                    $this->addFlash(
                        'notice',
                        'Your changes were saved!'
                    );

                }

                return $this->redirectToRoute('app_manage_networks');

            }

        }

        return $this->render('app/pages/network/networkEdit.html.twig', ['title' => $result['name'], 'form' => $form->createView()]);

    }

    /**
     * @param \AppBundle\Entity\Network $objectOriginal
     * @param \AppBundle\Entity\Network $updatedObject
     */
    private function setChangedFwRules(\AppBundle\Entity\Network $objectOriginal, \AppBundle\Entity\Network $updatedObject){


        $apiFwRules = $this->container->get('cs_api_fw_rules');

        if($objectOriginal->getDefaultFwPolicy() != $updatedObject->getDefaultFwPolicy()){

            $apiFwRules->setNetworkFwPolicy($updatedObject->getId(), $updatedObject->getDefaultFwPolicy());

        }

        if($objectOriginal->getAcceptFwRules()->toArray() != $updatedObject->getAcceptFwRules() &&
            $objectOriginal->getDenyFwRules()->toArray() != $updatedObject->getDenyFwRules()){

            $data = $updatedObject->getAcceptFwRules();

            $arrayAccept = $this->convertToArray($data, FwRules::RELATED_POLICY_DROP);

            $data = $updatedObject->getDenyFwRules();

            $arrayDeny = $this->convertToArray($data, FwRules::RELATED_POLICY_ACCEPT);

            $array = array_merge($arrayAccept, $arrayDeny);

            $apiFwRules->addNetworkFwRules($objectOriginal->getId(), $array);

        }else{

            if($objectOriginal->getAcceptFwRules()->toArray() != $updatedObject->getAcceptFwRules()){

                $data = $updatedObject->getAcceptFwRules();

                $arrayAccept = $this->convertToArray($data, FwRules::RELATED_POLICY_DROP);

                $arrayDenyOrig = $this->convertToArray($objectOriginal->getDenyFwRules()->toArray(), FwRules::RELATED_POLICY_ACCEPT);

                $array = array_merge($arrayAccept, $arrayDenyOrig);

                $apiFwRules->addNetworkFwRules($objectOriginal->getId(), $array);

            }

            if($objectOriginal->getDenyFwRules()->toArray() != $updatedObject->getDenyFwRules()){

                $data = $updatedObject->getDenyFwRules();

                $arrayDeny = $this->convertToArray($data, FwRules::RELATED_POLICY_ACCEPT);

                $arrayAcceptOrig = $this->convertToArray($objectOriginal->getAcceptFwRules()->toArray(), FwRules::RELATED_POLICY_DROP);

                $array = array_merge($arrayDeny, $arrayAcceptOrig);

                $apiFwRules->addNetworkFwRules($objectOriginal->getId(), $array);

            }

        }

    }

    /**
     * @param $networkId
     * @param $apiFwRules
     * @return NetworkFwRules|object
     */
    private function setFwRules($networkId, array $apiFwRules){

        $array = array();

        foreach ($apiFwRules as $apiFwRule){

            $fwRule = clone $this->container->get('network_fw_rule');
            $fwRule->setNetworkId($networkId);
            $fwRule->setEnabled(($apiFwRule['enabled'] == '1') ? true : false);
            $fwRule->setProtocol($apiFwRule['protocol']);
            $fwRule->setPorts($apiFwRule['ports']);
            $array[] = $fwRule;

        }
        return $array;
    }

    /**
     * @param $data
     * @param $policy
     * @return array
     */
    private function convertToArray($data, $policy){

        $items = array();
        $item = array();

        foreach ($data as $dataItem){
            $item['enabled'] = ($dataItem->getEnabled() == true) ? '1' : '0';
            $item['related_policy'] = (string) $policy;
            $item['protocol'] = $dataItem->getProtocol();
            $item['ports'] = ($dataItem->getPorts() == null) ? '' : $dataItem->getPorts();

            $items[] = $item;
        }

        return $items;

    }

}
