<?php

/*
 * For the full copyright and license information, please view the LICENSE file
 * that was distributed with this source code.
 */

namespace AppBundle\Entity;

use FOS\UserBundle\Model\User as BaseUser;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints\Length;
use Symfony\Component\Validator\Constraints\Regex;
use Symfony\Component\Validator\Mapping\ClassMetadata;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Constraints\Email;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;

/**
 * @ORM\Entity
 * @ORM\Table(name="app_user")
 */
class User extends BaseUser
{

    const ROLE_DEFAULT = 'ROLE_OBSERVER';
    const ROLE_DEVICE_ADMIN = 'ROLE_DEVICE_ADMIN';
    const ROLE_NET_MANAGER = 'ROLE_NET_MANAGER';
    const ROLE_NET_ADMIN = 'ROLE_NET_ADMIN';
    const ROLE_SYSTEM_ADMIN = 'ROLE_SYSTEM_ADMIN';

    /**
     * ...
     * @UniqueEntity(fields={"email"}
     */

    /**
     * @ORM\Id
     * @ORM\Column(type="integer")
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(type="string", nullable=true, length=50)
     */
    protected $firstname;

    /**
     * @ORM\Column(type="string", nullable=true, length=50)
     */
    protected $lastname;

    /**
     * User constructor.
     */
    public function __construct(){

        parent::__construct();

    }

    /**
     * @return mixed
     */
    public function getFirstName(){

        return $this->firstname;

    }

    /**
     * @param $firstName
     * @return $this
     */
    public function setFirstName($firstName){

        $this->firstname = $firstName;

        return $this;

    }

    /**
     * @return mixed
     */
    public function getLastName(){

        return $this->lastname;

    }

    /**
     * @param $lastName
     * @return $this
     */
    public function setLastName($lastName){

        $this->lastname = $lastName;

        return $this;

    }

    /** Set a list of rules to use when validating an instance of this Class
    @param Symfony\Component\Validator\Mapping\ClassMetadata $metadata */
    public static function loadValidatorMetadata(ClassMetadata $metadata) {

        $metadata->addPropertyConstraint('email', new Length(array('max'=>50)));
        $metadata->addPropertyConstraint('email', new NotBlank());
        $metadata->addPropertyConstraint('email', new Email());

        $metadata->addConstraint(new UniqueEntity(array(
                "fields" => "email",
                "message" => "This email address is already in use")
        ));

        $metadata->addPropertyConstraint('username', new Length(array('min'=>5)));
        $metadata->addPropertyConstraint(
            'username'
            , new Regex(array(
                'pattern' => '/^[a-zA-Z0-9_]+$/'
            )
        ));

        $metadata->addConstraint(new UniqueEntity(array(
                "fields" => "username",
                "message" => "This username is already in use")
        ));

    }


}
