<?php

namespace AppBundle\Form;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\CollectionType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\OptionsResolver\OptionsResolver;

class Network extends AbstractType
{

    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options){

        $isNew = $options['isNew'];

        $builder->add('name', TextType::class, array (
                    'attr' => array('autofocus' => null),
                    'horizontal_label_class' => 'col-md-2',
                    'horizontal_input_wrapper_class' => "col-lg-4",
                )
            );

        if(!$isNew){

            $builder->add('defaultFwPolicy',ChoiceType::class, array(
                'horizontal_input_wrapper_class' => "col-lg-2",
                'horizontal_label_class' => 'col-md-2',
                'widget_form_group_attr' => array('class'=>'form-group fw-policy'),
                'label' => 'FW rules default policy',
                'choices' => array(
                    'Allow' => '0',
                    'Deny' => '1',
                ),

            ))
                ->add('acceptFwRules',CollectionType::class, array(
                    'entry_type' => NetworkFwRule::class,
                    'label' => 'Firewall Allow Rules',
                    'widget_type' => 'inline',
                    'allow_add' => true,
                    'allow_delete' => true, // should render default button, change text with widget_remove_btn
                    'prototype' => true,
                    'horizontal' => true,
                    //'label_render' => false,
                    'horizontal_label_div_class' => 'col-md-8',
                    'horizontal_input_wrapper_class' => "col-lg-8",
                    'widget_form_group_attr' => array('class'=>'form-group fw-policy accept'),
                    'widget_add_btn' => array(
                        'label' => "Add rule",
                        "icon" => "plus",
                        'attr' => array('class' => 'btn btn-success'),
                        'horizontal_wrapper_div' => array('class' => "col-lg-10  col-lg-offset-3"),
                    ),
                    'entry_options' => array(
                        'horizontal_input_wrapper_class' => "col-lg-8 col-lg-offset-2",
                        'horizontal_label_offset_class' => '',
                        'label_render' => false,

                        'widget_remove_btn' => array(
                            'label' => "Remove",
                            'horizontal_wrapper_div' => array(
                                'class' => "col-lg-2 network-fw-remove"
                            ),
                            "icon" => "remove",
                            'attr' => array('class' => 'btn btn-danger'),
                            'wrapper_div'  => false,
                        ),
                    )
                ))
                ->add('denyFwRules',CollectionType::class, array(
                    'entry_type' => NetworkFwRule::class,
                    'label' => 'Firewall Deny Rules',
                    'widget_type' => 'inline',
                    'allow_add' => true,
                    'allow_delete' => true, // should render default button, change text with widget_remove_btn
                    'prototype' => true,
                    'horizontal' => true,
                    //'label_render' => false,
                    'horizontal_label_div_class' => 'col-md-8',
                    'horizontal_input_wrapper_class' => "col-lg-8",
                    'widget_form_group_attr' => array('class'=>'form-group fw-policy drop'),
                    'widget_add_btn' => array(
                        'label' => "Add rule",
                        "icon" => "plus",
                        'attr' => array('class' => 'btn btn-success'),
                        'horizontal_wrapper_div' => array('class' => "col-lg-10  col-lg-offset-3"),
                    ),
                    'entry_options' => array(
                        'horizontal_input_wrapper_class' => "col-lg-8 col-lg-offset-2",
                        'horizontal_label_offset_class' => '',
                        'label_render' => false,

                        'widget_remove_btn' => array(
                            'label' => "Remove",
                            'horizontal_wrapper_div' => array(
                                'class' => "col-lg-2 network-fw-remove"
                            ),
                            "icon" => "remove",
                            'attr' => array('class' => 'btn btn-danger'),
                            'wrapper_div'  => false,
                        ),
                    )
                ));


        }

        $builder->add('save', SubmitType::class, ['label' => 'Save', 'attr' => array('class' => 'btn btn-success', 'first-button')])
            ->add('apply', SubmitType::class, ['label' => 'Apply', 'attr' => array('class' => 'btn btn-primary', 'last-button')]);

    }

    /**
     * @param OptionsResolver $resolver
     */
    public function configureOptions(OptionsResolver $resolver){

        $resolver->setRequired('isNew');

    }

}
