<?php

namespace AppBundle\Form;

use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\SubmitType;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;

class UserFormType extends \Symfony\Component\Form\AbstractType
{

    /**
     * @param FormBuilderInterface $builder
     * @param array $options
     */
    public function buildForm(FormBuilderInterface $builder, array $options){

        $user = $options['data'];

        if($user->getUserName() != 'admin'){

            $builder
                ->add('username', TextType::class, [
                    'label' => 'Username',
                ]);

        }

        $builder->add('firstName', TextType::class, [
                'label' => 'First Name',
            ])
            ->add('lastName', TextType::class, [
                'label' => 'Last Name',
            ])
            ->add('email', EmailType::class, [
                'label' => 'Email',
            ])
            ->add('password', RepeatedType::class, array(
                'type' => PasswordType::class,
                'invalid_message' => 'The password fields must match.',
                'options' => array('attr' => array('class' => 'password-field')),
                'required' => ($builder->getData()->getId() == null) ? true : false,
                'first_options' => array('label' => 'Password'),
                'second_options' => array('label' => 'Repeat Password'),
        ));

        if($user->getUserName() != 'admin'){

            $builder->add('roles', ChoiceType::class, array(
                'label' => 'Roles',
                'choices' => $options['roles'],
                'multiple' => true,
                'expanded' => true
            ))
            ->add('enabled', CheckboxType::class, [
                'label' => 'Enabled',
                'required' => false,
                //'attr' => array('checked' => 'checked'),
            ]);

        }

        $builder
        ->add('save', SubmitType::class, [
            'label' => 'Save',
        ]);

    }

    /**
     * @param OptionsResolver $resolver
     */
    public function configureOptions(OptionsResolver $resolver){

        $resolver->setDefaults(array(
            'roles' => null,
        ));

    }

}
